@@ -3,7 +3,7 @@ package Class::Tag;
 #use 5.006; 
 
 use strict qw[vars subs];
-$Class::Tag::VERSION = '0.03';  
+$Class::Tag::VERSION = '0.07';  
 
 =head1 NAME
 
@@ -11,20 +11,24 @@ Class::Tag - programmatically label (mark) classes, methods, roles and modules w
 
 =head1 Warning
 
-Any specific interface that Class::Tag exposes may change (as it already did) untill version 1.0 is reached. 
+Any specific interface that Class::Tag exposes may change (as it already did) until version 1.0 is reached. 
 
 =head1 SYNOPSIS
 
-Directly using Class::Tag as tagger class: 
+The syntax of Class::Tag usage is an interaction of B<tag>, B<tagger> (class) and B<target> (class): tagger applies tag to a target class. Names of tagger class (except Class::Tag itself) and tag can be chosen almost freely (with usual restrictions) to be read together as (subject and predicate in a) self-explanatory English sentence, with question semantics (e.g. in conditionals) optionally toggled by direct/indirect method call notation. The following synopsis illustrates.
 
-	package Foo;
-	use Class::Tag 'tagged'; 
+Directly using Class::Tag as tagger: 
+
+	package   Foo;
+	use Class::Tag 'tagged'; # tagging Foo class with 'tagged' tag
 	tag Class::Tag 'tagged'; # same, but at run-time
 
+	# query 'tagged' tag on the Foo and Bar...
 	require Foo; # required before next check
 	Class::Tag->tagged('Foo'); # true
 	Class::Tag->tagged('Bar'); # false
 
+	# remove 'tagged' tag from Foo...
 	#no   Class::Tag 'tagged'; # at compile-time, so will not work - instead...
 	untag Class::Tag 'tagged'; # at run-time
 	Class::Tag->tagged('Foo'); # false
@@ -36,15 +40,24 @@ If no tags are given, the 'is' tag is assumed:
 	use Class::Tag 'is'; # same
 	use Class::Tag ();   # no tagging
 
-New tagger class can be created by simply tagging package with special 'tagger_class' tag using Class::Tag or any other tagger class, and then declaring specific tags or any tag to be used with that new tagger class. Tags declaration is done by tagger class tagging itself with those specific tags or special 'AUTOLOAD' tag:
+New tagger class can be created by simply tagging package with special 'tagger_class' tag using either Class::Tag or any other tagger class, and then declaring specific tags to be used with that new tagger class. Declaration of specific tag is done by new tagger class applying this tag to itself. Declaring special 'AUTOLOAD' tag this way effectively declares that any tag can be used with new tagger class:
 
-	package Awesome;                # new tagger class
-	use  Class::Tag 'tagger_class'; # must be before following declarations
-	use     Awesome 'specific_tag'; # declare 'specific_tag' for use
-	use     Awesome 'AUTOLOAD';     # declares that any tag can be used
+	{
+		# this block can be used as "inline" tagger class definition
+		# or contents of this block can be loaded from Awesome.pm
+
+		package Awesome;                # new tagger class
+		use  Class::Tag 'tagger_class'; # must be before following declarations
+		use     Awesome 'specific_tag'; # declares 'specific_tag' for use
+		use     Awesome 'AUTOLOAD';     # declares that any tag can be used
+
+		1;
+	}
 
 	Class::Tag->tagger_class('Awesome'); # true
 
+Note that Awesome class is not required to be loaded from .pm file with use() or require(), it can be simply defined as above at any point in the code prior to using it as tagger class. Such tagger class definition is referred to as "inline" tagger class.
+
 The Class::Tag itself is somewhat similar to the following implicit declaration:
 
 	package Class::Tag; 
@@ -84,7 +97,7 @@ Using default 'is' tag:
 Using tags 'class' and 'pureperl': 
 
 	package Foo; 
-	# tagging class Foo with tags 'class' and 'pureperl' of Awesome tagger class...
+	# tagger class Foo with tags 'class' and 'pureperl' of Awesome tagger class...
 	use Awesome  'class';
 	use Awesome              'pureperl';
 	use Awesome  'class',    'pureperl';       # same
@@ -135,26 +148,30 @@ Inheriting tags, using for example the default 'is' tag:
 
 =head1 DESCRIPTION
 
-Sometimes it is necessary to programmatically tag modules and classes with some meta-tags (arbitrary labels or key/value pairs) to be able to assert that you deal with proper classes (module), methods and roles. Such need typically arises for plug-in modules, application component modules, complex class inheritance hierarchies, etc. 
+Sometimes it is necessary to programmatically tag modules and classes with some meta-data tags (arbitrary labels or key/value pairs) to be able to assert that you deal with proper classes (modules), methods and roles. Such need typically arises for plug-in modules, application component modules, complex class inheritance hierarchies, etc. 
 
 Class::Tag allows programmatically label (mark) classes and modules with arbitrary inheritable tags (key/value pairs) without collision with methods/attributes/functions of the class/module and query those tags on arbitrary classes and modules.
 
-By design, Class::Tag is a generalized framework for meta information (tags) about inheritable behaviors. Inheritable behaviors that can have meta-data tags attached include methods, classes, roles, etc. Since tags are meta information about inheritable behaviors, tags themselves are inheritable (i.e. remain always "attached" to those behaviors). One example of the meta-tag is the class name itself with tag's (boolean) value returned by isa(). Another simple meta-tag example is method name with its value returned by can(). Yet another meta-tag example is the role name, with tag's value supposed to be returned by DOES(). But classes, methods and roles may also have other meta-tags apart from their names. In particular, Class::Tag can easily be used to implement method attributes, and even multi-layer method attributes, for example:
+By design, Class::Tag is a generalized framework for managing meta information (tags) about inheritable behaviors. Inheritable behaviors that can have meta-data tags attached include methods, classes, roles, etc. Tags are by necessity inheritable, as they need to be inherited together with behaviors they are supposed to describe.
 
-	package   Zoo;
-	use Meta2 foo => { author => 'metadoo', doc => 'is dead-simple' };
-	use Meta  foo => { is => 'ro', returns => 'boolean' };
-	sub       foo { 1 }
+Simple example of the meta-data tag is a class name, with tag's (boolean) value returned by isa(). Another simple example of meta-data tag is a method name, with its value returned by can(). Yet another meta-data tag example is a role name, with tag's value supposed to be returned by DOES(). But classes, methods and roles may also have other meta-data tags apart from their names. In particular, Class::Tag can easily be used to implement method attributes and even multiple "layers" of method attributes, for example:
 
-Such use opens possibilities for meta-programming and introspection. For example, method may access its own meta-data as follows:
+	package Zoo;
+
+	sub       foo    { 1 }
+	use Meta  foo => { is => 'ro', returns => 'boolean' };              # 1-st "meta-layer"
+	use Meta2 foo => { author => 'metadoo', doc => 'is dead-simple' };  # 2-nd "meta-layer"
+
+Such use opens possibilities for meta-programming and introspection. For example, method can access its own meta-data as follows:
 
 	sub foo { Meta->foo( ref($_[0])||$_[0] ) }
+	sub foo { Meta->foo(     $_[0]         ) } # nearly (but not exactly) same
 
-Technically, Class::Tag is the constructor of special variety of class/object attributes that are orthogonal to (isolated from) conventional attributes/methods of the class. Being the same and being orthogonal at the same time is what required to be good carrier of meta information about inheritable behavior. And use of tagger classes is a way to extend and partition class's namespace into meaningful orthogonal domains, as well as to extend the notion of the meta-tag in the domain-specific way.
+Technically, Class::Tag is the constructor for special variety of class/object attributes that are orthogonal to (isolated from) conventional attributes/methods of the class. Being the same and being orthogonal at the same time is what required to be good carrier of meta information about inheritable behavior. And use of tagger classes is a way to extend and partition class's namespace into meaningful orthogonal domains, as well as to extend the notion of the meta-data tag in the domain-specific way.
 
-=head1 Isolated (orthogonal) meta-domains
+=head1 Tagger classes
 
-Class::Tag itself serves as tagger class, and each tagger class is a "constructor" for other tagger classes, either loadable or inlined. Each tagger class brings separate meta-tags namespace that is orthogonal (isolated) to that of other tagger classes. The use of specific meta-tags namespace usually involves specific semantics. Together specific isolated meta-tags namespace and associated semantics are referred to as "meta-domain".
+Class::Tag itself serves as tagger class, and each tagger class is a "constructor" for other tagger classes, either loadable or inlined.
 
 The use() of tagger class looks as if it exports chosen named tags into packages, but in fact it doesn't - tagger class itself provides samename accessor methods for those tags. As a result, tag names can be arbitrary without risk of collision, so that together with name of tagger class they can be selected to read somewhat meaningful (see examples in L</"SYNOPSIS">) in the problem area domain that uses that specific tagger.
 
@@ -162,7 +179,7 @@ The use() of tagger class looks as if it exports chosen named tags into packages
 
 See L</"SYNOPSIS"> for description of new tagger class creation. Tagger class can be created "inline", without using separate .pm file for it.
 
-The value of 'tagger_class' tag is reserved for special use in the future, so it should not be used for anything to avoid collision with future versions.
+The value of 'tagger_class' tag is reserved for special use in the future, so it should not be used for anything to avoid incompatibility with future versions.
 
 =head2 Tagger class benefits
 
@@ -174,13 +191,13 @@ There are a few reasons to use multiple tagger classes in addition to or instead
 
 Name of the tagger class can be chosen to read naturally and meaningful, in either direct or indirect method call notations i.e. reversing order of tagger and tag names (doubling readability options), with semantically meaningful tags used in the context of given application or problem area domain.
 
-=item Collision with Class::Tag guts
+=item Restricted tagspace
 
-The original Class::Tag tagger class is not empty, so that not every tag can be used. In contrast, any empty package can be used as tagger classes (but tag(), untag() and Perl's specials, like import(), can(), etc. are still reserved).
+The original Class::Tag tagger class allows to use any tag (except tag(), untag() and Perl's specials, like import(), can(), etc. are still reserved). In contrast, custom tagger classes may allow only specific tags to be used.
 
-=item Orthogonality of tags
+=item Isolated (orthogonal) tagspace
 
-Each tagger class has its own orthogonal tags namespace, so that same tags of different tagger classes do not collide:
+Each tagger class has its own orthogonal tags namespace (tagspace), so that same tags of different tagger classes do not collide:
 
 	package Awesome;
 	use  Class::Tag 'tagger_class';
@@ -206,9 +223,9 @@ Since tagger class tags upon use() and classes usually do not export anything, i
 
 However, making existing (non-empty) class/module a tagger class requires care to not collide with methods of that class - Class::Tag will raise an exception when such collision happens. It is better not to declare 'AUTOLOAD' for such tagger class.
 
-=item Separate namespace and semantics domain
+=item Meta-data domains
 
-Tagger class is a class intended for defining, managing and documenting specific meta-tags and domain-specific meta-tags namespace. In particular, tagger class is an ideal place where to document tags from that namespace.
+Tags of different tagger classes are intended to be dedicated to defining, managing and documenting different meta-data domains. It can be meta-data associated with specific module, application, problem, algorithm, etc. In particular, tagger class is an ideal place where to document its tags.
 
 =back
 
@@ -220,8 +237,8 @@ In addition, values of declaration tags can be used to modify behavior of tags a
 
 	package Awesome;                             # new tagger class
 	use  Class::Tag 'tagger_class';              # must be before following declarations
-	use     Awesome  specific_tag => \&accessor; # declare 'specific_tag' for use with \&accessor
-	use     Awesome  AUTOLOAD     => \&ACCESSOR; # declares that any tag can be used and uses \&accessor for all of them
+	use     Awesome  specific_tag => \&accessor; # use \&accessor for 'specific_tag' 
+	use     Awesome  AUTOLOAD     => \&ACCESSOR; # use \&ACCESSOR for any tag
 
 	Awesome->specific_tag( $class_or_obj, @args); # is equivalent to...
 	&accessor('Awesome',   $class_or_obj, @args); 
@@ -239,17 +256,21 @@ The Awesome class in above code may also be replaced with object of Awesome clas
 
 =head1 Traditional alternatives
 
-There are two natural alternative solutions: classes-as-tags and methods-as-tags. The classes-as-tags solution is using universal isa() method to see if class has specific parent and effectively using specific parent classes as tags. However, using parent classes just as tags is a limited solution since @ISA is used for different things and better be used for those things exclusively to avoid interferences. 
+There are three natural alternative solutions: classes-as-tags, roles-as-tags and methods-as-tags. The classes-as-tags solution uses universal isa() method to see if class has specific parent, it effectively uses specific parent classes as tags. However, using parent classes just as tags is a limited solution since @ISA is used for different things and better be used for those things exclusively to avoid interferences. 
 
-Using methods-as-tags approach is about defining and using specific methods as tags. It is way better then classes-as-tags, but but if specific method-tag need to be queried on unknown class/module, the following problems may arise: 
+Using roles as tags do not involve modifying @ISA, but this approach relies on using single shared congested namespace, which means possibility of accidental collision, unless you specifically choose unnatural names (long, prefixed, capitalized, etc.) that are unlikely to collide or use unique names of existing modules as tags, which is an overkill in many cases.
+
+Moreover, classes-as-tags and roles-as-tags solutions do not allow using values for tags (unless properly overridden). 
+
+Using methods-as-tags approach is about defining and using specific methods as tags. This approach is far better than classes-as-tags and roles-as-tags, but if specific method-tag need to be queried on unknown class/module, the following problems may arise: 
 
 =over
 
 =item Name collision
 
-It may be that class/module have defined samename method/attribute by coincidence. Possibility of collision is considerable for short readable names (like 'is'), especially for undocumented tags that are used internally and in case of subclassing. To avoid collision method-tags usually have some unique prefix and may be in upper-case and/or starting with '_', etc. The typical solution is prefixing name of some module as unique identifier, and this is exactly what Class::Tag does in its own way.
+It may be that class/module have defined samename method/attribute by coincidence. Possibility of collision is considerable for short readable names (like 'is'), especially for undocumented tags that are used internally and in case of subclassing. To avoid collision method-tags usually have some unique prefix and may be in upper-case and/or starting with '_', etc. The typical solution is using name of some module as unique suffix/prefix, and this is exactly what Class::Tag does in its own flexible way:
 
-	Foo->Awesome_is;
+	Foo->is_Awesome;
 
 	Awesome->is('Foo');
 
@@ -270,28 +291,32 @@ Class::Tag solve this problem.
 
 =item Tagging
 
-Tagging using tag methods is essentially defining an attribute. For tagging classes only it is simple enough, but for tagging blesses-hash objects ends up in writing accessor, so it requires use of some attributes construction module, of which Class::Tag is essentially the one:
+Tagging is essentially defining an attribute. Applying tag to class is simple enough, but applying tag to blessed-hash objects ends up in writing accessor, so it requires use of some attributes construction module, of which Class::Tag is essentially the one:
 
 	package Foo;
 	bless $obj = {}, 'Foo';
 
-	sub Foo::Awesome_is    { 'old_value' }; # compile-time tagging
-	*Foo::Awesome_is = sub { 'old_value' };
-	*Foo::Awesome_is = sub { 'new_value' };
+	sub Foo::is_Awesome    { 'old_value' }; # compile-time tagging
+	*Foo::is_Awesome = sub { 'old_value' };
+	*Foo::is_Awesome = sub { 'new_value' };
 	# tagging object instance of the class...
-	sub  Foo::Awesome_is { @_ > 1 ? $_[0]->{Awesome_is} = $_[1] : $_[0]->{Awesome_is} }
-	$obj->Awesome_is('new_value');
+	sub  Foo::is_Awesome { @_ > 1 ? $_[0]->{is_Awesome} = $_[1] : $_[0]->{is_Awesome} }
+	$obj->is_Awesome('new_value');
 
-	use Awesome    is => 'old_value'; # compile-time tagging
+	use Awesome   is  => 'old_value'; # compile-time tagging
 	is  Awesome 'Foo' => 'old_value';
 	is  Awesome 'Foo' => 'new_value';
 	is  Awesome $obj  => 'new_value';
 
-except Class::Tag's default accessor implement copy-on-write tags on blessed-hash object instances (and simple tag inheritance by instances otherwise), rather than simplistic accessor in above alternative.
+except Class::Tag's default accessor implements copy-on-write tag values on blessed-hash object instances (and simple tag inheritance from class for blessed-non-hashes), rather than simplistic accessor in above alternative.
 
 =back
 
-Class::Tag solves these problems by moving tag creation and tag accessors to "tagger classes".
+Class::Tag solves these problems by moving tag constructors and accessors to tagger class, which is far more predictable and controlled environment.
+
+=head1 SEE ALSO
+
+The Class::DOES module provide the ability to use DOES() for tagging classes with role names - see discusssion in L</"Traditional alternatives">.
 
 =head1 SUPPORT
 
